/*
 * Programmieren fuer Physiker, SS 2023, Blatt 11, Aufg. 29
 *
 * Kochsche Schneeflocke, Version mit Klasse fuer 2d-Vektoren
 */

#include "CImg.h"
using namespace cimg_library;


// --- Kleine Klasse fuer 2d Punkte und Vektoren
// x to the right, y down!
class vec2d
{
public:
    vec2d (double x_, double y_) : x(x_), y(y_) {} ;
    vec2d () {} ;
    vec2d operator+( vec2d op2) 
	{ return vec2d (x+op2.x, y+op2.y) ; } ;
    vec2d operator-( vec2d op2) 
	{ return vec2d (x-op2.x, y-op2.y) ; } ;
    vec2d flipright()  // rotate vec 90 degrees counterclockwise
	{ return vec2d (-y, x) ; } ;
    friend vec2d operator*( double a, vec2d p)  ; // scalar*vektor
    friend void drawline( CImg<unsigned char>& canvas, vec2d p1, vec2d p2) ;
private:
    double x,y ;
} ;

void drawline( CImg<unsigned char>& canvas, vec2d p1, vec2d p2) 
{
    const unsigned char white[] = { 255,255,255 };
    canvas.draw_line ( int(p1.x), int(p1.y), int(p2.x), int(p2.y), white);
}

vec2d operator*( double a, vec2d p) 
{ return vec2d (a*p.x, a*p.y) ; } ;


// --- Koch-Schneeflocke 
void koch_line( CImg<unsigned char>& canvas, 
		vec2d p1, vec2d p2, int depth)
{
    if (0==depth)
	drawline (canvas, p1, p2) ;
    else
    {
	// depth >= 1: divide line into four segments
	vec2d midpeak = p1 + 1.0/2.0*(p2-p1) 
	    + 1/(2.0*sqrt(3))*(p2-p1).flipright() ;
	koch_line ( canvas, p1, p1 + 1.0/3.0*(p2-p1), depth-1) ;
	koch_line ( canvas, p1 + 1.0/3.0*(p2-p1), midpeak, depth-1) ;
	koch_line ( canvas, midpeak, p1 + 2.0/3.0*(p2-p1), depth-1) ;
	koch_line ( canvas, p1 + 2.0/3.0*(p2-p1), p2, depth-1) ;
    }
    return ;
}

int main() {
    const int sizeX = 800 ;             // Fenstergroesse X
    const int sizeY = int(sizeX*0.85) ; // Fenstergroesse Y

    // explicitely give 4 points for the initial 3 line segments.
    const int numb_points = 4 ;
    const double cornX = 0.2*sizeX ;  // linke untere Ecke X
    const double cornY = 0.6*sizeX ;  // linke untere Ecke Y
    const double len = 0.6*sizeX ;    // Laenge einer Dreiecksseite
    vec2d pnt [numb_points] 
	= { vec2d(cornX,cornY), 
	    vec2d(cornX+len,cornY), 
	    vec2d(cornX+len/2.0,cornY-len*sqrt(3)/2.0), 
	    vec2d(cornX,cornY) } ;
    
    CImg<unsigned char> canvas( sizeX, sizeY, 1, 3);

    CImgDisplay window( canvas, "Koch-Schneeflocke");
 
    int depth = 0 ;
    while (!window.is_closed()) 
    {
	// draw the snowflake
	canvas.fill(0) ;
	for( int i=0; i<numb_points-1; i++) 
	    koch_line( canvas, pnt[i], pnt[i+1], depth) ;

	// and a text
	const unsigned char white[] = { 255,255,255 } ;	
	char depthtext[40] ;
	sprintf(depthtext, "Ersetzungstiefe: %d", depth) ;
	canvas.draw_text(10,10, depthtext,white) ;
	canvas.display(window) ;	
	
	// use left/right mouse button to change depth
	window.wait();
	if (1==window.button() && depth<8) depth++ ; 
	if (2==window.button() && depth>0) depth-- ; 
    }
}
